% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PeriodSPEI.R
\name{CST_PeriodSPEI}
\alias{CST_PeriodSPEI}
\title{Compute the Standardised Precipitation-Evapotranspiration Index}
\usage{
CST_PeriodSPEI(
  exp,
  exp_cor = NULL,
  time_dim = "syear",
  leadtime_dim = "time",
  memb_dim = "ensemble",
  lat_dim = "latitude",
  accum = 1,
  ref_period = NULL,
  params = NULL,
  pet_exp = NULL,
  pet_expcor = NULL,
  standardization = TRUE,
  cross_validation = FALSE,
  pet_method = "hargreaves",
  method = "parametric",
  distribution = "log-Logistic",
  param_error = -9999,
  handle_infinity = FALSE,
  return_params = FALSE,
  na.rm = FALSE,
  ncores = NULL
)
}
\arguments{
\item{exp}{A named list with the needed \code{s2dv_cube} objects containing 
the seasonal forecast experiment in the data element for each variable. 
Specific variables are needed for each method used in computing the 
Potential Evapotranspiration. See parameter 'pet_method'. The accepted 
variables are: 'tasmin' and 'tasmax', required for methods 'hargreaves' and 
'hargreaves_modified' and 'tas', required for method 'thornthwaite'. 
Variable 'prlr' is always needed. The units for temperature variables
('tasmin', 'tasmax' and 'tas') need to be in Celcius degrees; the units for 
precipitation ('prlr') need to be in mm/month.}

\item{exp_cor}{A named list with the needed \code{s2dv_cube} objects for each 
variable in which the quantile PeriodSPEI should be applied. If it is not 
specified, the PeriodSPEI is calculated from object 'exp'.}

\item{time_dim}{A character string indicating the name of the temporal 
dimension. By default, it is set to 'syear'.}

\item{leadtime_dim}{A character string indicating the name of the temporal 
dimension. By default, it is set to 'time'.}

\item{memb_dim}{A character string indicating the name of the dimension in 
which the ensemble members are stored. When set it to NULL, threshold is 
computed for individual members.}

\item{lat_dim}{A character string indicating the name of the latitudinal 
dimension. By default it is set by 'latitude'.}

\item{accum}{An integer value indicating the number of months for the 
accumulation for each variable. When it is greater than 1, the result will 
be filled with NA until the accum time_dim dimension number due to the 
accumulation to previous months.}

\item{ref_period}{A list with two numeric values with the starting and end 
points of the reference period used for computing the index. The default 
value is NULL indicating that the first and end values in data will be 
used as starting and end points.}

\item{params}{A multidimensional array with named dimensions for computing the 
SPEI. This option overrides computation of fitting parameters. It needs 
to have the following dimensions: same leadtime dimension of 'exp' 
(specified in 'leadtime_dim'); time dimension of length 1 (specified in 
'time_dim'); and a dimension named 'coef' with the length of the 
coefficients needed for the used distribution (for 'Gamma' coef dimension is 
of lenght 2, for 'log-Logistic' or 'PearsonIII' is of length 3). It can't 
have member dimension (specified in 'memb_dim').}

\item{pet_exp}{A multidimensional array containing the Potential
EvapoTranspiration data of 'exp'. It must have the same dimensions of the 
variable 'prlr' of 'exp'. If it is NULL it is calculated using the provided 
variables with specified 'pet_method'. It is NULL by default.}

\item{pet_expcor}{A multidimensional array containing the Potential
EvapoTranspiration data of 'exp_cor'. It must have the same dimensions of 
the variable 'prlr' of 'exp_cor'. If it is NULL it is calculated using the 
provided variables with specified 'pet_method'. It is NULL by default.}

\item{standardization}{A logical value indicating wether the standardization 
is computed.}

\item{cross_validation}{A logical value indicating if cross validation is 
done (TRUE), or not (FALSE). It only will be used when 'exp_cor' and 
is not provided. It is FALSE by default.}

\item{pet_method}{A character string indicating the method used to compute 
the potential evapotranspiration. The accepted methods are:
'hargreaves' and 'hargreaves_modified', that require the data to have 
variables tasmin and tasmax; and 'thornthwaite', that requires variable 
'tas'.}

\item{method}{A character string indicating the standardization method used. 
If can be: 'parametric' or 'non-parametric'. It is set to 'parametric' by 
default.}

\item{distribution}{A character string indicating the name of the distribution 
function to be used for computing the SPEI. The accepted names are: 
'log-Logistic' and 'Gamma'. It is set to 'log-Logistic' by default. The 
'Gamma' method only works when only precipitation is provided and other 
variables are 0 because it is positive defined (SPI indicator).}

\item{param_error}{A numeric value with the error accepted.}

\item{handle_infinity}{A logical value wether to return Infinite values (TRUE)
or not (FALSE).}

\item{na.rm}{A logical value indicating whether NA values should be removed 
from data. It is FALSE by default. If it is FALSE and there are NA values, 
(if standardization is TRUE) all values of other dimensions except time_dim 
and leadtime_dim will be set to NA directly. On the other hand, if it is 
TRUE, if the data from other dimensions except time_dim and leadtime_dim is 
not reaching 4 values, it is not enough values to estimate the parameters 
and the result will include NA.}

\item{ncores}{An integer value indicating the number of cores to use in 
parallel computation.}
}
\value{
An 's2dv_cube' object containing the SPEI multidimensional array in 
element \code{data}. If 'exp_cor' is provided, only results from 'exp_cor' 
will be provided. The parameters of the standardization will only be returned 
if 'return_params' is TRUE. The SPEI will only be computed if 
'standardization' is TRUE. If 'standardization' is FALSE, only the climatic 
water balance (precipitation minus evapotranspiration) will be returned. The 
resultant arrays will have the same dimensions as the initial input data. The 
other elements in the 's2dv_cube' will be updated with the combined 
information of the input data arrays.
}
\description{
Calculation of the Standardised Precipitation-Evapotranspiration Index (SPEI) 
that is a multiscalar drought index based on climatic data. It can be used for 
determining the onset, duration and magnitude of drought conditions with 
respect to normal conditions in a variety of natural and managed systems such 
as crops, ecosystems, rivers, water resources, etc. The idea behind the SPEI 
is to compare the highest possible evapotranspiration with the current water 
availability. The SPEI uses the monthly (or weekly) difference between 
precipitation and potential evapotranspiration. This represents a simple 
climatic water balance which is calculated at different time scales to obtain 
the SPEI. This function is build to work be compatible with other tools in 
that work with 's2dv_cube' object class. The input data must be this object 
class. If you don't work with 's2dv_cube', see PeriodSPEI.
}
\details{
Next, some specifications for the calculation of this indicator will be 
discussed. On the one hand, the model to be used to calculate potential 
evapotranspiration is specified with the pet_method parameter (hargreaves, 
hargraves modified or thornwhite). On the other hand, to choose the time scale 
in which you want to accumulate the SPEI (SPEI3, SPEI6...) is done using the 
accum parameter, where you must indicate the number of time steps you want to 
accumulate throughout leadtime_dim. Since the accumulation is done for the 
elapsed time steps, there will be no complete accumulations until reaching the 
time instant equal to the value of the parameter. For this reason, in the 
result, we will find that for the dimension where the accumulation has been 
carried out, the values of the array will be NA since they do not include 
complete accumulations. Also, there is a parameter to specify if the 
standardization that can be TRUE or FALSE. If it is TRUE, the data is fit to a 
probability distribution to transform the original values to standardized 
units that are comparable in space and time and at different SPEI time scales. 
The na.rm parameter is a logical parameter used to decide whether to remove 
the NA values from the data before doing the calculation. It must be taken 
into account that if na.rm == FALSE and there is some NA value in the specific 
coordinates which the SPEI is computed, standardization cannot be carried out 
for those coordinates and therefore, the result will be filled with NA for the 
specific coordinates. However, when na.rm == TRUE,  if the amount of data for 
those specific coordinates is smaller than 4, it will not be possible to carry 
out because we will not have enough data and the result will be also filled 
with NAs for that coordinates.
}
\examples{
dims <- c(var = 1, sday = 1, sweek = 1, syear = 6, time = 3, 
         latitude = 2, longitude = 1, ensemble = 25)
dimscor <- c(var = 1, sday = 1, sweek = 1, syear = 1, time = 3, 
            latitude = 2, longitude = 1, ensemble = 15)

dates_exp <- as.POSIXct(c(paste0(2010:2015, "-08-16"), 
                         paste0(2010:2015, "-09-15"), 
                         paste0(2010:2015, "-10-16")), 'UTC')
dim(dates_exp) <- c(sday = 1, sweek = 1, syear = 6, time = 3)
dates_expcor <- as.POSIXct(c(paste0(2020, "-08-16"), paste0(2020, "-09-15"), 
                            paste0(2020, "-10-16")), 'UTC')
dim(dates_expcor) <- c(sday = 1, sweek = 1, syear = 1, time = 3)
lat <- c(40,40.1)

exp_tasmax <- array(rnorm(900, 27.73, 5.26), dim = dims)
exp_tasmin <- array(rnorm(900, 14.83, 3.86), dim = dims)
exp_prlr <- array(rnorm(900, 21.19, 25.64), dim = dims)

expcor_tasmax <- array(rnorm(540, 29.03, 5.67), dim = dimscor)
expcor_tasmin <- array(rnorm(540, 15.70, 4.40), dim = dimscor)
expcor_prlr <- array(rnorm(540, 15.62, 21.38), dim = dimscor)

exp <- list('tasmax' = exp_tasmax, 'tasmin' = exp_tasmin, 'prlr' = exp_prlr)
exp_cor <- list('tasmax' = expcor_tasmax, 'tasmin' = expcor_tasmin, 
               'prlr' = expcor_prlr)

exp <- lapply(exp, CSTools::s2dv_cube, coords = list(latitude = lat), 
             Dates = dates_exp)
exp_cor <- lapply(exp_cor, CSTools::s2dv_cube, coords = list(latitude = lat), 
                 Dates = dates_expcor)

res <- CST_PeriodSPEI(exp = exp, exp_cor = exp_cor)

}
